@@ -19,9 +19,10 @@ from turbogears import config
 from turbogears.decorator import weak_signature_decorator
 from turbogears import validators
 
-from tgwebservices import soap
+from tgwebservices import soap, iconv
 from tgwebservices.runtime import register, primitives, \
-                                  FunctionInfo, register_complex_type
+                                  FunctionInfo, register_complex_type, \
+                                  ctvalues
 
 config.update({"tg.empty_flash" : False})
 
@@ -58,24 +59,53 @@ def wsexpose(return_type=basestring):
         tgfunc = tgexpose("wsautoxml", accept_format="text/xml",
                           allow_json=True)(dict_wrapped_result(func))
         
-        def newfunc(*args, **kw):
+        def newfunc(self, **kw):
             if "tg_format" in kw:
                 if kw["tg_format"] == "json":
                     cherrypy.request.headers["Accept"] = "text/javascript"
                 else:
                     cherrypy.request.headers["Accept"] = "text/xml"
                 del kw["tg_format"]
+            
+            request = cherrypy.request
+            input_types = fi.input_types
+            
+            # the _tgws keyword is used when a web service system, such as
+            # SOAP, is calling the function rather than a call that is being
+            # made via direct URL traversal.
+            if "_tgws" in kw:
+                webservice = True
+                del kw["_tgws"]
+            else:
+                webservice = False
+            
+            if "xml_body" in kw:
+                kw = iconv.handle_xml_params(kw["xml_body"], input_types)
+            elif request.headers.get("Content-Type", "") \
+                 .startswith("text/xml"):
+                try:
+                    clen = int(request.headers.get('Content-Length')) or 0
+                    data = request.body.read(clen)
+                    body = et.fromstring(data)
+                except SyntaxError:
+                    raise validators.Invalid("Request XML is invalid", "", 
+                                             None)
+                kw = iconv.handle_xml_params(body, input_types)
+            elif request.headers.get("Content-Type", "") \
+                .startswith("application/json"):
+                clen = int(request.headers.get('Content-Length')) or 0
+                data = request.body.read(clen)
+                kw = iconv.handle_json_params(data, input_types)
+            else:
+                iconv.handle_keyword_params(kw, input_types)
+            
             try:
-                # the _tgws keyword is used when a web service system, such as
-                # SOAP, is calling the function rather than a call that is being
-                # made via direct URL traversal.
-                if "_tgws" in kw:
-                    del kw["_tgws"]
-                    return func(*args, **kw)
+                if webservice:
+                    return func(self, **kw)
                 else:
                     # when direct traversal is used, call the standard
                     # TurboGears expose mechanism.
-                    return tgfunc(*args, **kw)
+                    return tgfunc(self, **kw)
             except TypeError:
                 # handle the case where a bogus parameter was sent in to the
                 # function
@@ -107,14 +137,6 @@ def wsexpose(return_type=basestring):
         return newfunc
     return entangle
 
-def boolean_converter(value):
-    value = value.lower()
-    if value in ["", "false", "0", "no", "off"]:
-        return False
-    if value in ["true", "1", "yes", "on"]:
-        return True
-    raise validators.Invalid("%s is not a legal boolean value" % (value))
-
 def wsvalidate(*args, **kw):
     """Validates and converts incoming parameters. Also registers the 
     parameters used by the method for use by statically typed languages.
@@ -130,35 +152,11 @@ def wsvalidate(*args, **kw):
         # does. So, the parameters list is offset by one higher.
         for i in range(0, len(args)):
             argtype = args[i]
-            if argtype is bool:
-                argtype = boolean_converter
             input_types[fi.params[i]] = argtype
         
         input_types.update(kw)
         fi.input_types = input_types
-        
-        def newfunc(self, **kw):
-            # convert the input parameters to appropriate types
-            for key in kw:
-                if key in input_types:
-                    try:
-                        kw[key] = input_types[key](kw[key])
-                    except ValueError:
-                        raise validators.Invalid(
-                            "%s value for the '%s' parameter is not a "
-                            "valid %s" % (kw[key], key, 
-                                         input_types[key].__name__), 
-                            kw[key], None)
-                else:
-                    raise validators.Invalid(
-                        "%s is not a valid parameter (valid values are: %s)"
-                        % (key, input_types.keys()), key, None
-                    )
-            return func(self, **kw)
-        newfunc.__name__ = func.__name__
-        newfunc.__doc__ = func.__doc__
-        newfunc._ws_func_info = func._ws_func_info
-        return newfunc
+        return func
     return entangle
 
 class WebServicesController(object):
