@@ -5,6 +5,7 @@ import re
 import sys
 import traceback
 import time
+import logging
 
 import dispatch
 import cherrypy
@@ -16,6 +17,8 @@ from turbogears import controllers, expose as tgexpose, error_handler, \
 
 from tgwebservices.runtime import primitives, ctvalues, typedproperty, unsigned
 
+log = logging.getLogger("tgwebservices.soap")
+
 namespace_expr = re.compile(r'^\{.*\}')
 
 _simplevalues = set([int, long, basestring, str, unicode, float])
@@ -189,28 +192,24 @@ class SoapController(controllers.Controller):
         methodname = namespace_expr.sub("", methodname)
         request.soap_method = methodname
         
-        # pull out all of the parameters
-        params = {}
-        for paramelem in body.getchildren():
-            paramname = namespace_expr.sub("", paramelem.tag)
-            val = paramelem.text
-            if val is None:
-                val = ""
-            params[paramname] = val
-        
         try:
             method = self.wscontroller._ws_funcs[methodname]
         except KeyError:
             raise validators.Invalid("Unknown SOAP method: %s" %
                                      methodname, methodname, None)
         
+        fi = method._ws_func_info
+        input_types = fi.input_types
+        
+        params = {"_tgws" : True}
+        params["xml_body"] = body
+        
         # ensure that the requested method has actually been exposed
         assert hasattr(method, "exposed") and method.exposed == True
         
         # set the _tgws flag so that the TurboGears wrapping is not applied
         # in wsexpose. We want to manage the produced dictionary here, because
         # we need to pass additional information on to the soap template.
-        params["_tgws"] = True
         output = method(**params)
         data = dict(baseURL=self.wscontroller._ws_baseURL,
                     tns=self.tns,
