@@ -1,3 +1,5 @@
+import cStringIO as StringIO
+
 import cherrypy
 from turbogears import testutil
 
@@ -37,7 +39,7 @@ def test_error_handling():
     testutil.create_request("/somestrings?foo=1&tg_format=json")
     output = cherrypy.response.body[0]
     print output
-    assert output == """{"faultcode": "Client", "faultstring": "Unexpected parameter in function call (somestrings() got an unexpected keyword argument 'foo')"}"""
+    assert output == """{"faultcode": "Client", "faultstring": "foo is not a valid parameter (valid values are: [])"}"""
 
 class BooleanInput(WebServicesRoot):
     @wsexpose(bool)
@@ -63,4 +65,19 @@ def test_boolean_input():
     ]
     for test in tests:
         yield check, test[0], test[1]
-        
\ No newline at end of file
+
+def test_complex_input():
+    cherrypy.root = ComplexService("http://foo.bar.baz/")
+    
+    request = """{
+        "person" : {"name" : "Fred", "age" : 22}
+}"""
+    testutil.create_request("/tenyearsolder", rfile=StringIO.StringIO(request), 
+                            method="POST", 
+                            headers={"Content-Length" : str(len(request)),
+                                     "Content-Type" : 
+                                        "application/json; charset=utf-8"})
+    output = cherrypy.response.body[0]
+    print output
+    print cherrypy.root.last_person.age
+    assert cherrypy.root.last_person.age == 32
\ No newline at end of file
