@@ -292,8 +292,8 @@ def test_fault_for_invalid_input():
     cherrypy.root = MyService("http://foo.bar.baz")
     output = run_soap("times2", """<value>whaleblubber</value>""")
     print cherrypy.response.status
-    assert cherrypy.response.status == "500 Invalid Input"
     print output
+    assert cherrypy.response.status == "500 Invalid Input"
     assert output == """<Envelope xmlns="http://schemas.xmlsoap.org/soap/envelope/"><Body><Fault><faultcode>Client</faultcode><faultstring>whaleblubber value for the 'value' parameter is not a valid int</faultstring></Fault></Body></Envelope>"""
 
 def test_fault_for_exception():
@@ -431,3 +431,23 @@ def test_empty_parameter_is_empty_string():
     output = run_soap("say_hello", """<name></name>""")
     print output
     assert "fault" not in output
+
+class Person(object):
+    name = str
+    age = int
+
+class ComplexInput(WebServicesRoot):
+    @wsexpose()
+    @wsvalidate(Person)
+    def savePerson(self, p):
+        self.person = p
+    
+def test_complex_input_simple_object():
+    ci = ComplexInput("http://foo.bar.baz/")
+    cherrypy.root = ci
+    output = run_soap("savePerson", """<p>
+    <name>George</name>
+    <age>280</age>
+</p>""")
+    print output
+    assert isinstance(ci.person, Person)
\ No newline at end of file
